/*

  DEBUG_master_counter.ino

  Ira Hart  10/2020

  Sends all the 6 clock digits a count from 0 to 9 every 8 seconds.

  This master Arduino Nano sends master_commands to
  A slave Arduino nano controller.
  
  It sends position, direction and speed master_commands.
  TIt sends a calibration command and then 10 seconds later, it starts counting.

  The clock face is numbered 1 through 8. Number 8 at the top middle of the circle.
  1 is to the right of that and follows clockwise.

  Commands:
  0= IDLE,
  1= Calibrate,
  2= Arm_go_clockwise,
  3= Arm_go_counter_clockwise

  Whole Clock Arduino address pattern:

  00 03 06 09 12 15 18 21

  01 04 07 10 13 16 19 22

  02 05 08 11 14 17 20 23


  Clock Face pattern:

     8
   7   1
  6     2
   5   3
     4

*/

// libraries:
#include <Wire.h> 	  //for I2C communication:
#include <SerialCommand.h>

//For real time clock
#include "RTClib.h"
RTC_DS3231 rtc;


char daysOfTheWeek[7][12] = {"Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday"};

//Constants:
#define DEBUG_PRINT_ENABLE  1

byte slave_command_mot_0 	= 1; 	//calibrate
byte slave_next_position_mot_0 	= 8; 	//clock position 1->8
byte slave_command_mot_1 	= 1; 	//calibrate
byte slave_next_position_mot_1   = 8;  //clock position 1->8
byte MS_Hour   = 0;
byte LS_Hour   = 0;
byte MS_Minute   = 0;
byte LS_Minute   = 0;
byte digit_position_hour_minute = 0;
byte digit_position_offset = 0;
byte digit_position = 0;
byte hours = 0;
unsigned long slave_mS_delay 		= 0;
byte mot_0_inversion = 0;
byte mot_1_inversion = 0;
byte slave_address;

// Constant arrays for the clock arm positions.
// _0[] is for mot_0, [1] is for mot_1:
//                0  1  2  3  4  5
//                -----------------
byte zero_0[]  = {2, 4, 2, 4, 4, 6};
byte zero_1[]  = {4, 8, 8, 6, 8, 8};
//byte one_0[]   = {5, 5, 5, 4, 4, 4};
//byte one_1[]   = {5, 5, 5, 8, 8, 8};
byte one_0[]   = {5, 5, 5, 4, 4, 8};
byte one_1[]   = {5, 5, 5, 4, 8, 8};
byte two_0[]   = {2, 2, 2, 4, 6, 6};
byte two_1[]   = {2, 4, 8, 6, 8, 6};
byte three_0[] = {2, 2, 2, 4, 4, 6};
byte three_1[] = {2, 2, 2, 6, 8, 8};
//byte four_0[]  = {4, 2, 5, 4, 6, 4};
//byte four_1[]  = {8, 8, 5, 8, 8, 8};
byte four_0[]  = {4, 2, 5, 4, 6, 8};
byte four_1[]  = {4, 8, 5, 4, 8, 8};
byte five_0[]  = {2, 2, 2, 6, 4, 6};
byte five_1[]  = {4, 8, 2, 6, 6, 8};
byte six_0[]   = {2, 4, 2, 6, 4, 6};
byte six_1[]   = {4, 8, 8, 6, 6, 8};
//byte seven_0[] = {2, 5, 5, 4, 4, 4};
//byte seven_1[] = {6, 5, 5, 6, 8, 8};
byte seven_0[] = {2, 5, 5, 4, 4, 8};
byte seven_1[] = {6, 5, 5, 6, 8, 8};
byte eight_0[] = {2, 2, 2, 4, 6, 6};
byte eight_1[] = {4, 8, 8, 6, 8, 8};
byte nine_0[]  = {2, 2, 2, 4, 6, 6};
byte nine_1[]  = {4, 8, 2, 6, 8, 8};
byte blank_0[] = {5, 5, 5, 5, 5, 5};
byte blank_1[] = {5, 5, 5, 5, 5, 5};
// this is about 132 bytes total...


// Reverse direction if stepper motor runs backwards from factory:
// Constant arrays for motor inversion.  0 is normal, 1 is inverted.
//motor 0 = arm closest to face
//motor 1 = outside arm
//                            0  1  2  3  4  5   6  7  8  9 10 11  12 13 14 15 16 17  18 19 20 21 22 23
//                           ------------------------------------------------------------------------------
  int motor_0_inversion[]  = {0, 0, 0, 0, 0, 1,  0, 1, 0, 0, 0, 0,  0, 1, 1, 1, 0, 0,  1, 0, 0, 0, 0, 1}; //Inner
  int motor_1_inversion[]  = {0, 0, 0, 0, 0, 0,  0, 0, 0, 0, 1, 0,  0, 1, 0, 0, 0, 0,  1, 0, 0, 0, 0, 0}; //Outer

int now_hour;

//daylight savings time offset
int hour_offset_button = 0;

SerialCommand sCmd;     // The SerialCommand object for parsing strings

long motor;
long delaySeconds;

byte debug_digit_counter = 0;
unsigned long time_millis = 0;

//////////////////////////////////////////////
void setup()
//////////////////////////////////////////////
{
  Serial.begin(57600);//for debug printing to screen
  Wire.begin(); // join i2c bus (address optional for master)

  Serial.println("Entering Setup...");

 //RTC init code:
  if (! rtc.begin()) {
    Serial.println("Couldn't find RTC");
    while (1);
  }
  if (rtc.lostPower()) {
    Serial.println("RTC lost power, lets set the time!");
    // following line sets the RTC to the date & time this sketch was compiled
    rtc.adjust(DateTime(F(__DATE__), F(__TIME__)));
    // This line sets the RTC with an explicit date & time, for example to set
    // January 21, 2014 at 3am you would call:
    // rtc.adjust(DateTime(2014, 1, 21, 3, 0, 0));
  }


  //send calibrate command to all clocks.  Both arms will be at the top (8,8)
  delay(1000); // delay 1S to allow command to complete.
  for (int digit_position = 0; digit_position < 4; digit_position++) {
    for (int clock_face = 0; clock_face < 6; clock_face++) {
      slave_address = (clock_face + (digit_position * 6));
      transmit_to_slave(slave_address, 1, zero_0[clock_face], 1, zero_1[clock_face], 50, motor_0_inversion[slave_address], motor_1_inversion[slave_address]); //(address,cmd_0,nxt_pos_0,cmd_1,nxt_pos_1,dly,m0_inv,m1_inv)
      delay(1); // delay 1 mS to allow command to complete.
    } //end for loop
  } //end for loop

  //Delay x seconds to allow slave calibrate to complete.  
  delay(20000);

}//end of setup

//////////////////////////////////////////////
void loop() //main loop
//////////////////////////////////////////////
{
  //Delay 8 seconds
  //Call the routine to send the current count to the digits.
  //increment the count.
  

  if ((long)(millis() - time_millis) > 8000) { //after x seconds
    digitalClockTimeToClockFaces(); //send debug count to clocks.


    if (debug_digit_counter == 9) {
      debug_digit_counter = 0;
    }
    else {
      debug_digit_counter++;
    }

    
    time_millis = millis();//grab current time
  }//end if long wait loop

}//end main loop



//////////////////////////////////////////////
void digitalClockTimeToClockFaces(void)   //Print out the time to clock faces:
//////////////////////////////////////////////
{
  
    DateTime now = rtc.now(); //get time from RTC
  
    //move hours to temporary variable
    now_hour = now.hour();
  
    //add or subtract hour offset from pushbutton to adjust daylight savings time etc.
    now_hour = now_hour + hour_offset_button;
  
  //convert RTC Hours (0->23) to 12hour time (not military) MS and LS digits:
    hours = ((now_hour + 11) % 12 + 1);
  
//    MS_Hour = hours / 10; //e.g. 12 hours/10 = 1   
//    LS_Hour = hours % 10; //e.g. 12 hours mod 10 = 2
//  
//    //convert RTC minutes (0->59) to MS and LS digits:
//    MS_Minute = now.minute() / 10; //e.g. 51 minutes/10 = 5
//    LS_Minute = now.minute() % 10; //e.g. 51 minutes mod 10 = 1


//  //From debug counter in Main. print debug counter to each digit:
  MS_Hour = debug_digit_counter;
  LS_Hour = debug_digit_counter;
  MS_Minute = debug_digit_counter;
  LS_Minute = debug_digit_counter;


    //DEBUG print time to terminal window
    if (DEBUG_PRINT_ENABLE == 1) {
      digitalClockDebugPrint();
    }


  //Example: 12:53: MS_hour = 1, LS_Hour = 2, MS_minute = 5, LS_minute = 3


  //In each case below, for a digit (MS_hour etc.) write array to 6 clock faces, hand0=x_0 and hand1=x_1 (where x is for example one.)

  //Current digit starting at MS left most MS_Hour digit.
  for (int digit_position = 0; digit_position < 4; digit_position++) {

    //find digit_postion_hour_minute = , , ,
    switch (digit_position) {
      case 0:
        digit_position_hour_minute = MS_Hour;
        break;
      case 1:
        digit_position_hour_minute = LS_Hour;
        break;
      case 2:
        digit_position_hour_minute = MS_Minute;
        break;
      case 3:
        digit_position_hour_minute = LS_Minute;
        break;
    }//end case


    //find top left clock# of current digit
    switch (digit_position) {
      case 0:
        digit_position_offset = 0;//top left clock# of digit
        break;
      case 1:
        digit_position_offset = 6;//top left clock# of digit
        break;
      case 2:
        digit_position_offset = 12;//top left clock# of digit
        break;
      case 3:
        digit_position_offset = 18;//top left clock# of digit
        break;
    }//end case


    //each case here will write to 6 clocks (2 motors each) with it's arm digits from the above constant arrays.
    switch (digit_position_hour_minute) { //0->9 possible number for each digit position
      case 0:
        for (int clock_face = 0; clock_face < 6; clock_face++) {
          slave_address = (clock_face + digit_position_offset);
          transmit_to_slave(slave_address, 2, zero_0[clock_face], 2, zero_1[clock_face], 50, motor_0_inversion[slave_address], motor_1_inversion[slave_address]); //(address,cmd_0,nxt_pos_0,cmd_1,nxt_pos_1,dly,m0_inv,m1_inv)
        } //end for loop
        break;

      case 1:
        for (int clock_face = 0; clock_face < 6; clock_face++) {
          slave_address = (clock_face + digit_position_offset);
          transmit_to_slave(slave_address, 2, one_0[clock_face], 2, one_1[clock_face], 50, motor_0_inversion[slave_address], motor_1_inversion[slave_address]); //(address,cmd_0,nxt_pos_0,cmd_1,nxt_pos_1,dly,m0_inv,m1_inv)
        } //end for loop
        break;

      case 2:
        for (int clock_face = 0; clock_face < 6; clock_face++) {
          slave_address = (clock_face + digit_position_offset);
          transmit_to_slave(slave_address, 2, two_0[clock_face], 2, two_1[clock_face], 50, motor_0_inversion[slave_address], motor_1_inversion[slave_address]); //(address,cmd_0,nxt_pos_0,cmd_1,nxt_pos_1,dly,m0_inv,m1_inv)
        } //end for loop
        break;

      case 3:
        for (int clock_face = 0; clock_face < 6; clock_face++) {
          slave_address = (clock_face + digit_position_offset);
          transmit_to_slave(slave_address, 2, three_0[clock_face], 2, three_1[clock_face], 50, motor_0_inversion[slave_address], motor_1_inversion[slave_address]); //(address,cmd_0,nxt_pos_0,cmd_1,nxt_pos_1,dly,m0_inv,m1_inv)
        } //end for loop
        break;

      case 4:
        for (int clock_face = 0; clock_face < 6; clock_face++) {
          slave_address = (clock_face + digit_position_offset);
          transmit_to_slave(slave_address, 2, four_0[clock_face], 2, four_1[clock_face], 50, motor_0_inversion[slave_address], motor_1_inversion[slave_address]); //(address,cmd_0,nxt_pos_0,cmd_1,nxt_pos_1,dly,m0_inv,m1_inv)
        } //end for loop
        break;

      case 5:
        for (int clock_face = 0; clock_face < 6; clock_face++) {
          slave_address = (clock_face + digit_position_offset);
          transmit_to_slave(slave_address, 2, five_0[clock_face], 2, five_1[clock_face], 50, motor_0_inversion[slave_address], motor_1_inversion[slave_address]); //(address,cmd_0,nxt_pos_0,cmd_1,nxt_pos_1,dly,m0_inv,m1_inv)
        } //end for loop
        break;

      case 6:
        for (int clock_face = 0; clock_face < 6; clock_face++) {
          slave_address = (clock_face + digit_position_offset);
          transmit_to_slave(slave_address, 2, six_0[clock_face], 2, six_1[clock_face], 50, motor_0_inversion[slave_address], motor_1_inversion[slave_address]); //(address,cmd_0,nxt_pos_0,cmd_1,nxt_pos_1,dly,m0_inv,m1_inv)
        } //end for loop
        break;

      case 7:
        for (int clock_face = 0; clock_face < 6; clock_face++) {
          slave_address = (clock_face + digit_position_offset);
          transmit_to_slave(slave_address, 2, seven_0[clock_face], 2, seven_1[clock_face], 50, motor_0_inversion[slave_address], motor_1_inversion[slave_address]); //(address,cmd_0,nxt_pos_0,cmd_1,nxt_pos_1,dly,m0_inv,m1_inv)
        } //end for loop
        break;

      case 8:
        for (int clock_face = 0; clock_face < 6; clock_face++) {
          slave_address = (clock_face + digit_position_offset);
          transmit_to_slave(slave_address, 2, eight_0[clock_face], 2, eight_1[clock_face], 50, motor_0_inversion[slave_address], motor_1_inversion[slave_address]); //(address,cmd_0,nxt_pos_0,cmd_1,nxt_pos_1,dly,m0_inv,m1_inv)
        } //end for loop
        break;

      case 9:
        for (int clock_face = 0; clock_face < 6; clock_face++) {
          slave_address = (clock_face + digit_position_offset);
          transmit_to_slave(slave_address, 2, nine_0[clock_face], 2, nine_1[clock_face], 50, motor_0_inversion[slave_address], motor_1_inversion[slave_address]); //(address,cmd_0,nxt_pos_0,cmd_1,nxt_pos_1,dly,m0_inv,m1_inv)
        } //end for loop
        break;


      default:
        break;
    }//end case

  } //end for (int digit_position = 0; digit_position < 4; digit_position)
} //end digitalClockTimeToClockFaces

////////////////////////////////////////////////
void transmit_to_slave(int address, int command_mot_0, int next_position_mot_0, int command_mot_1, int next_position_mot_1, unsigned long mS_delay, byte mot_0_inversion, byte mot_1_inversion)
////////////////////////////////////////////////
{
  Wire.beginTransmission(address);   // transmit to device #
  Wire.write(command_mot_0);            // sends one byte
  Wire.write(next_position_mot_0);      // sends one byte
  Wire.write(command_mot_1);            // sends one byte
  Wire.write(next_position_mot_1);      // sends one byte
  Wire.write(mot_0_inversion);      // sends one byte
  Wire.write(mot_1_inversion);      // sends one byte
  Wire.endTransmission();       // stop transmitting
  delay(mS_delay); //short delay to allow transmit to finish.
} //end transmit_to_slave


////////////////////////////////////////////////
void digitalClockDebugPrint(void) // I2C Real Time Clock debug
////////////////////////////////////////////////
{
  DateTime now = rtc.now(); //get time from RTC

  Serial.print(now.year(), DEC);
  Serial.print('/');
  Serial.print(now.month(), DEC);
  Serial.print('/');
  Serial.print(now.day(), DEC);
  Serial.print(" (");
  Serial.print(daysOfTheWeek[now.dayOfTheWeek()]);
  Serial.print(") ");
  Serial.print(now.hour(), DEC);
  Serial.print(':');
  Serial.print(now.minute(), DEC);
  Serial.print(':');
  Serial.print(now.second(), DEC);
  Serial.println();

  Serial.print("DEBUG!: ");
  Serial.println();
  Serial.print("MS_Hour: ");
  Serial.print(MS_Hour);
  Serial.print("  LS_Hour: ");
  Serial.print(LS_Hour);
  Serial.print("  MS_Minute: ");
  Serial.print(MS_Minute);
  Serial.print("  LS_Minute: ");
  Serial.print(LS_Minute);
  Serial.println();
} // end digitalClockDebugPrint
